@@ -57,6 +57,13 @@ core_opts = [
                       "or negative integer means no limit")),
     cfg.HostAddressOpt('host', default=utils.get_hostname(),
                        help=_("The hostname Tacker is running on")),
+    cfg.BoolOpt('use_credential_encryption', default=False,
+                help=_("Enable to encrypt the credential")),
+    cfg.StrOpt('keymanager_type', default="barbican",
+               help=_("The type of keymanager to use when the "
+                      "'use_credential_encryption' option is True")),
+    cfg.StrOpt('crypt_key_dir', default="/etc/tacker/crypt/fernet_keys",
+               help=_("Dir.path to store fernet_keys"))
 ]
 
 core_cli_opts = [
@@ -82,13 +89,8 @@ oslo_messaging.set_transport_defaults(control_exchange='tacker')
 
 
 def set_db_defaults():
-    # Update the default QueuePool parameters. These can be tweaked by the
-    # conf variables - max_pool_size, max_overflow and pool_timeout
-    db_options.set_defaults(
-        cfg.CONF,
-        connection='sqlite://',
-        max_pool_size=10,
-        max_overflow=20, pool_timeout=10)
+    # Call to register `database` option to cfg.CONF
+    db_options.set_defaults(cfg.CONF)
 
 
 set_db_defaults()
@@ -128,7 +130,7 @@ def load_paste_app(app_name):
         raise cfg.ConfigFilesNotFoundError(
             config_files=[cfg.CONF.api_paste_config])
     config_path = os.path.abspath(config_path)
-    LOG.info("Config paste file: %s", config_path)
+    LOG.debug("Config paste file: %s", config_path)
 
     try:
         app = deploy.loadapp("config:%s" % config_path, name=app_name)
