@@ -26,8 +26,8 @@ from keystoneauth1 import exceptions as ks_exc
 from oslo_log import log as logging
 from oslo_utils import uuidutils
 from tacker.common import utils
-from tacker.sol_refactored.api import prometheus_plugin_validator as validator
 from tacker.sol_refactored.api.schemas import prometheus_plugin_schemas
+from tacker.sol_refactored.api import validator
 from tacker.sol_refactored.common import config as cfg
 from tacker.sol_refactored.common import exceptions as sol_ex
 from tacker.sol_refactored.common import fm_alarm_utils
@@ -505,9 +505,9 @@ class PrometheusPluginPm(PrometheusPluginPmBase, mon_base.MonitoringPlugin):
                 f"doesn't match pmJob.")
             raise sol_ex.PrometheusPluginSkipped()
 
-    @validator.schema(prometheus_plugin_schemas.AlertMessage)
+    @validator.schema_nover(prometheus_plugin_schemas.AlertMessage)
     def _alert(self, request, body):
-        result = []
+        result = {}
         context = request.context
         datetime_now = datetime.datetime.now(datetime.timezone.utc)
         for alert in body['alerts']:
@@ -529,7 +529,7 @@ class PrometheusPluginPm(PrometheusPluginPmBase, mon_base.MonitoringPlugin):
                     pm_job, object_instance_id, sub_object_instance_id)
                 value = self.convert_measurement_unit(metric, value)
 
-                result.append({
+                entry = {
                     'objectType': pm_job.objectType,
                     'objectInstanceId': object_instance_id,
                     'subObjectInstanceId': sub_object_instance_id,
@@ -538,17 +538,22 @@ class PrometheusPluginPm(PrometheusPluginPmBase, mon_base.MonitoringPlugin):
                         'timeStamp': datetime_now,
                         'value': value
                     }]
-                })
+                }
+                if pm_job_id in result:
+                    result[pm_job_id].append(entry)
+                else:
+                    result[pm_job_id] = [entry]
+
             except sol_ex.PrometheusPluginSkipped:
                 pass
         if len(result) > 0:
             if self.notification_callback:
-                # every job_id in body['alerts'] has same id
-                self.notification_callback(context, {
+                # Call ConductorV2
+                self.notification_callback(context, [{
                     'id': uuidutils.generate_uuid(),
                     'jobId': pm_job_id,
-                    'entries': result,
-                })
+                    'entries': entries
+                } for pm_job_id, entries in result.items()])
         return result
 
     def decompose_metrics_vnfc(self, pm_job):
@@ -762,7 +767,7 @@ class PrometheusPluginThreshold(PrometheusPluginPmBase,
             crossing_direction = "DOWN"
         return crossing_direction
 
-    @validator.schema(prometheus_plugin_schemas.AlertMessage)
+    @validator.schema_nover(prometheus_plugin_schemas.AlertMessage)
     def _alert(self, request, body):
         result = []
         context = request.context
@@ -1100,7 +1105,7 @@ class PrometheusPluginFm(PrometheusPlugin, mon_base.MonitoringPlugin):
             return [new_alarm]
         raise sol_ex.PrometheusPluginSkipped()
 
-    @validator.schema(prometheus_plugin_schemas.AlertMessage)
+    @validator.schema_nover(prometheus_plugin_schemas.AlertMessage)
     def _alert(self, request, body):
         now = datetime.datetime.now(datetime.timezone.utc)
         result = []
@@ -1153,7 +1158,7 @@ class PrometheusPluginAutoHealing(PrometheusPlugin, mon_base.MonitoringPlugin):
         self.rpc.enqueue_auto_heal_instance(
             context, vnf_instance_id, vnfc_info_id)
 
-    @validator.schema(prometheus_plugin_schemas.AlertMessage)
+    @validator.schema_nover(prometheus_plugin_schemas.AlertMessage)
     def _alert(self, request, body):
         context = request.context
         alerts = (alert for alert in body['alerts'] if
@@ -1225,7 +1230,7 @@ class PrometheusPluginAutoScaling(PrometheusPlugin, mon_base.MonitoringPlugin):
     def default_callback(self, context, vnf_instance_id, scaling_param):
         self.rpc.trigger_scale(context, vnf_instance_id, scaling_param)
 
-    @validator.schema(prometheus_plugin_schemas.AlertMessage)
+    @validator.schema_nover(prometheus_plugin_schemas.AlertMessage)
     def _alert(self, request, body):
         context = request.context
         alerts = (alert for alert in body['alerts'] if
