@@ -94,9 +94,11 @@ class VnfPmControllerV2(sol_wsgi.SolAPIController):
     def __init__(self):
         self.nfvo_client = nfvo_client.NfvoClient()
         self.endpoint = CONF.v2_vnfm.endpoint
-        self._pm_job_view = vnfpm_view.PmJobViewBuilder(self.endpoint)
+        self._pm_job_view = vnfpm_view.PmJobViewBuilder(self.endpoint,
+            CONF.v2_vnfm.vnfpm_pmjob_page_size)
         self._pm_threshold_view = (
-            vnfpm_view.PmThresholdViewBuilder(self.endpoint))
+            vnfpm_view.PmThresholdViewBuilder(self.endpoint,
+                CONF.v2_vnfm.vnfpm_pmthreshold_page_size))
         cls = plugin.get_class(
             CONF.prometheus_plugin.performance_management_package,
             CONF.prometheus_plugin.performance_management_class)
@@ -200,16 +202,8 @@ class VnfPmControllerV2(sol_wsgi.SolAPIController):
                                     location=location)
 
     def index(self, request):
-        filter_param = request.GET.get('filter')
-        if filter_param is not None:
-            filters = self._pm_job_view.parse_filter(filter_param)
-        else:
-            filters = None
-
-        # validate_filter
-        selector = self._pm_job_view.parse_selector(request.GET)
-        page_size = CONF.v2_vnfm.vnfpm_pmjob_page_size
-        pager = self._pm_job_view.parse_pager(request, page_size)
+        filters, selector, pager = self._pm_job_view.parse_query_params(
+            request)
         pm_job = pm_job_utils.get_pm_job_all(request.context,
                                              marker=pager.marker)
         resp_body = self._pm_job_view.detail_list(pm_job, filters,
@@ -241,12 +235,12 @@ class VnfPmControllerV2(sol_wsgi.SolAPIController):
             subsc_utils.test_notification(
                 pm_job, subsc_utils.NOTIFY_TYPE_PM)
 
-        with context.session.begin(subtransactions=True):
+        with context.session.begin():
             pm_job.update(context)
 
-        pm_job_modifications = objects.PmJobModificationsV2(
-            callbackUri=pm_job.callbackUri,
-        )
+        pm_job_modifications = objects.PmJobModificationsV2()
+        if body.get("callbackUri"):
+            setattr(pm_job_modifications, "callbackUri", body["callbackUri"])
         resp = pm_job_modifications.to_dict()
 
         return sol_wsgi.SolResponse(200, resp,
@@ -278,15 +272,15 @@ class VnfPmControllerV2(sol_wsgi.SolAPIController):
     def allowed_content_types(self, action):
         if action in {'update', 'update_threshold'}:
             # Content-Type of Modify request shall be
-            # 'application/mergepatch+json' according to SOL spec.
+            # 'application/merge-patch+json' according to SOL spec.
             # But 'application/json' and 'text/plain' is OK for backward
             # compatibility.
-            return ['application/mergepatch+json', 'application/json',
+            return ['application/merge-patch+json', 'application/json',
                     'text/plain']
         return ['application/json', 'text/plain']
 
     def allowed_accept(self, action):
-        return ['application/json', 'application/mergepatch+json',
+        return ['application/json', 'application/merge-patch+json',
                 'text/plain']
 
     def supported_api_versions(self, action):
@@ -372,12 +366,8 @@ class VnfPmControllerV2(sol_wsgi.SolAPIController):
                                     location=location)
 
     def index_threshold(self, request):
-        filter_param = request.GET.get('filter')
-        filters = (self._pm_threshold_view.parse_filter(filter_param)
-                   if filter_param else None)
-
-        page_size = CONF.v2_vnfm.vnfpm_pmthreshold_page_size
-        pager = self._pm_threshold_view.parse_pager(request, page_size)
+        filters, _, pager = self._pm_threshold_view.parse_query_params(
+            request)
         pm_job = pm_threshold_utils.get_pm_threshold_all(request.context,
                                                          marker=pager.marker)
         resp_body = self._pm_threshold_view.detail_list(pm_job, filters,
@@ -415,11 +405,13 @@ class VnfPmControllerV2(sol_wsgi.SolAPIController):
             pm_threshold.authentication = subsc_utils.get_subsc_auth(
                 body.get("authentication"))
 
-        with context.session.begin(subtransactions=True):
+        with context.session.begin():
             pm_threshold.update(context)
 
-        pm_threshold_modifications = objects.ThresholdModificationsV2(
-            callbackUri=pm_threshold.callbackUri)
+        pm_threshold_modifications = objects.ThresholdModificationsV2()
+        if body.get("callbackUri"):
+            setattr(
+                pm_threshold_modifications, "callbackUri", body["callbackUri"])
         resp = pm_threshold_modifications.to_dict()
         return sol_wsgi.SolResponse(200, resp,
                                     version=api_version.CURRENT_PM_VERSION)
@@ -439,3 +431,7 @@ class VnfPmControllerV2(sol_wsgi.SolAPIController):
 
         return sol_wsgi.SolResponse(204, None,
                                     version=api_version.CURRENT_PM_VERSION)
+
+    def set_default_to_response(self, result, action):
+        result.headers.setdefault('version', api_version.CURRENT_PM_VERSION)
+        result.headers.setdefault('accept-ranges', 'none')
