@@ -582,12 +582,18 @@ def alert_event_resolved(inst_id, pod_name):
     }
 
 
-def update_alarm():
+def update_alarm_acknowledged():
     return {
         "ackState": "ACKNOWLEDGED"
     }
 
 
+def update_alarm_unacknowledged():
+    return {
+        "ackState": "UNACKNOWLEDGED"
+    }
+
+
 def terminate_vnf_min():
     # Omit except for required attributes
     # NOTE: Only the following cardinality attributes are set.
