@@ -18,6 +18,10 @@ import time
 
 from tacker.tests.functional.sol_kubernetes_v2 import base_v2
 from tacker.tests.functional.sol_kubernetes_v2 import paramgen
+from tacker.tests.functional.sol_kubernetes_v2 import zipgen
+from tacker.tests import utils
+
+WAIT_LCMOCC_UPDATE_TIME = 3
 
 
 class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
@@ -26,16 +30,12 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
     def setUpClass(cls):
         super(VnfLcmHelmTest, cls).setUpClass()
 
-        cur_dir = os.path.dirname(__file__)
+        cls.helm_vnfd_id, zip_path = zipgen.test_helm_instantiate()
+        cls.helm_pkg = cls.upload_vnf_package(zip_path)
 
-        test_helm_instantiate_path = os.path.join(
-            cur_dir, "samples/test_helm_instantiate")
-        cls.vnf_pkg_1, cls.vnfd_id_1 = cls.create_vnf_package(
-            test_helm_instantiate_path)
-
-        test_helm_change_vnf_pkg_path = os.path.join(
-            cur_dir, "samples/test_helm_change_vnf_pkg")
-        cls.vnf_pkg_2, cls.vnfd_id_2 = cls.create_vnf_package(
+        test_helm_change_vnf_pkg_path = utils.test_sample(
+            "functional/sol_kubernetes_v2/test_helm_change_vnf_pkg")
+        cls.new_pkg, cls.new_vnfd_id = cls.create_vnf_package(
             test_helm_change_vnf_pkg_path)
         cls.helm_vim_id = cls.get_k8s_vim_id(use_helm=True)
 
@@ -43,8 +43,8 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
     def tearDownClass(cls):
         super(VnfLcmHelmTest, cls).tearDownClass()
 
-        cls.delete_vnf_package(cls.vnf_pkg_1)
-        cls.delete_vnf_package(cls.vnf_pkg_2)
+        cls.delete_vnf_package(cls.helm_pkg)
+        cls.delete_vnf_package(cls.new_pkg)
 
     def setUp(self):
         super(VnfLcmHelmTest, self).setUp()
@@ -96,7 +96,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
             '_links'
         ]
         create_req = paramgen.test_helm_instantiate_create(
-            self.vnfd_id_1)
+            self.helm_vnfd_id)
         resp, body = self.create_vnf_instance(create_req)
         self.assertEqual(201, resp.status_code)
         self.check_resp_headers_in_create(resp)
@@ -104,8 +104,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         inst_id = body['id']
 
         # check usageState of VNF Package
-        usage_state = self.get_vnf_package(self.vnf_pkg_1)['usageState']
-        self.assertEqual('IN_USE', usage_state)
+        self.check_package_usage(self.helm_pkg, 'IN_USE')
 
         # 2. Instantiate a VNF instance
         if not use_register_vim:
@@ -217,21 +216,20 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         vnfc_resource_infos = body['instantiatedVnfInfo']['vnfcResourceInfo']
         before_vdu2_ids = [vnfc_info['id'] for vnfc_info in vnfc_resource_infos
                            if vnfc_info['vduId'] == 'VDU2']
-        change_vnfpkg_req = paramgen.helm_change_vnfpkg(self.vnfd_id_2)
+        change_vnfpkg_req = paramgen.helm_change_vnfpkg(self.new_vnfd_id)
         resp, body = self.change_vnfpkg(inst_id, change_vnfpkg_req)
         self.assertEqual(202, resp.status_code)
         self.check_resp_headers_in_operation_task(resp)
 
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_complete(lcmocc_id)
-        time.sleep(3)
+        time.sleep(WAIT_LCMOCC_UPDATE_TIME)
 
         # check usageState of VNF Package
-        usage_state = self.get_vnf_package(self.vnf_pkg_1).get('usageState')
-        self.assertEqual('NOT_IN_USE', usage_state)
+        self.check_package_usage(self.helm_pkg, 'NOT_IN_USE')
+
         # check usageState of VNF Package
-        usage_state = self.get_vnf_package(self.vnf_pkg_2).get('usageState')
-        self.assertEqual('IN_USE', usage_state)
+        self.check_package_usage(self.new_pkg, 'IN_USE')
 
         # 11. Show VNF instance
         additional_inst_attrs = [
@@ -259,12 +257,8 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_complete(lcmocc_id)
 
-        # wait a bit because there is a bit time lag between lcmocc DB
-        # update and terminate completion.
-        time.sleep(3)
-
         # 13. Delete a VNF instance
-        resp, body = self.delete_vnf_instance(inst_id)
+        resp, body = self.exec_lcm_operation(self.delete_vnf_instance, inst_id)
         self.assertEqual(204, resp.status_code)
         self.check_resp_headers_in_delete(resp)
 
@@ -273,15 +267,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         self.assertEqual(404, resp.status_code)
 
         # check usageState of VNF Package
-        usage_state = self.get_vnf_package(self.vnf_pkg_1).get('usageState')
-        self.assertEqual('NOT_IN_USE', usage_state)
-
-    def _put_fail_file(self, operation):
-        with open(f'/tmp/{operation}', 'w'):
-            pass
-
-    def _rm_fail_file(self, operation):
-        os.remove(f'/tmp/{operation}')
+        self.check_package_usage(self.helm_pkg, 'NOT_IN_USE')
 
     def test_instantiate_rollback(self):
         """Test LCM operations with all attributes set
@@ -298,14 +284,14 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
 
         # 1. Create a new VNF instance resource
         create_req = paramgen.test_helm_instantiate_create(
-            self.vnfd_id_1)
+            self.helm_vnfd_id)
         resp, body = self.create_vnf_instance(create_req)
         self.assertEqual(201, resp.status_code)
         self.check_resp_headers_in_create(resp)
         inst_id = body['id']
 
         # 2. Instantiate a VNF instance => FAILED_TEMP
-        self._put_fail_file('instantiate_end')
+        self.put_fail_file('instantiate_end')
         instantiate_req = paramgen.helm_instantiate(
             self.auth_url, self.bearer_token, self.ssl_ca_cert)
         resp, body = self.instantiate_vnf_instance(inst_id, instantiate_req)
@@ -314,7 +300,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
 
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_failed_temp(lcmocc_id)
-        self._rm_fail_file('instantiate_end')
+        self.rm_fail_file('instantiate_end')
 
         # 3. Show VNF instance
         resp, body = self.show_vnf_instance(inst_id)
@@ -355,7 +341,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
 
         # 1. Create a new VNF instance resource
         create_req = paramgen.test_helm_instantiate_create(
-            self.vnfd_id_1)
+            self.helm_vnfd_id)
         resp, body = self.create_vnf_instance(create_req)
         self.assertEqual(201, resp.status_code)
         self.check_resp_headers_in_create(resp)
@@ -382,7 +368,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         self.assertEqual(1, len(vdu2_ids_0))
 
         # 4. Scale out a VNF instance => FAILED_TEMP
-        self._put_fail_file('scale_end')
+        self.put_fail_file('scale_end')
         scale_out_req = paramgen.helm_scale_out()
         resp, body = self.scale_vnf_instance(inst_id, scale_out_req)
         self.assertEqual(202, resp.status_code)
@@ -390,7 +376,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
 
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_failed_temp(lcmocc_id)
-        self._rm_fail_file('scale_end')
+        self.rm_fail_file('scale_end')
 
         # 5. Rollback scale out
         resp, body = self.rollback_lcmocc(lcmocc_id)
@@ -416,12 +402,8 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_complete(lcmocc_id)
 
-        # wait a bit because there is a bit time lag between lcmocc DB
-        # update and terminate completion.
-        time.sleep(3)
-
         # 8. Delete a VNF instance
-        resp, body = self.delete_vnf_instance(inst_id)
+        resp, body = self.exec_lcm_operation(self.delete_vnf_instance, inst_id)
         self.assertEqual(204, resp.status_code)
         self.check_resp_headers_in_delete(resp)
 
@@ -461,7 +443,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
             '_links'
         ]
         create_req = paramgen.test_helm_instantiate_create(
-            self.vnfd_id_1)
+            self.helm_vnfd_id)
         resp, body = self.create_vnf_instance(create_req)
         self.assertEqual(201, resp.status_code)
         self.check_resp_headers_in_create(resp)
@@ -469,8 +451,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         inst_id = body['id']
 
         # check usageState of VNF Package
-        usage_state = self.get_vnf_package(self.vnf_pkg_1)['usageState']
-        self.assertEqual('IN_USE', usage_state)
+        self.check_package_usage(self.helm_pkg, 'IN_USE')
 
         # 2. Instantiate a VNF instance
         instantiate_req = paramgen.helm_instantiate(
@@ -499,7 +480,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
 
         # 4. Change Current VNF Package => FAILED_TEMP
         change_vnfpkg_req = paramgen.helm_error_handling_change_vnfpkg(
-            self.vnfd_id_2)
+            self.new_vnfd_id)
         resp, body = self.change_vnfpkg(inst_id, change_vnfpkg_req)
         self.assertEqual(202, resp.status_code)
         self.check_resp_headers_in_operation_task(resp)
@@ -514,8 +495,7 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         self.wait_lcmocc_rolled_back(lcmocc_id)
 
         # check usageState of VNF Package
-        usage_state = self.get_vnf_package(self.vnf_pkg_2).get('usageState')
-        self.assertEqual('NOT_IN_USE', usage_state)
+        self.check_package_usage(self.new_pkg, 'NOT_IN_USE')
 
         # 6. Show VNF instance
         additional_inst_attrs = [
@@ -542,12 +522,8 @@ class VnfLcmHelmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_complete(lcmocc_id)
 
-        # wait a bit because there is a bit time lag between lcmocc DB
-        # update and terminate completion.
-        time.sleep(3)
-
         # 8. Delete a VNF instance
-        resp, body = self.delete_vnf_instance(inst_id)
+        resp, body = self.exec_lcm_operation(self.delete_vnf_instance, inst_id)
         self.assertEqual(204, resp.status_code)
         self.check_resp_headers_in_delete(resp)
 
